# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base
  helper :all # include all helpers, all the time
  #helper_method :iphone_request?
  
  helper_method :google_account_domain
  helper_method :logged_in?
  protect_from_forgery # See ActionController::RequestForgeryProtection for details


  before_filter :authorize #, :except=>[:index]
  
  def logged_in?
    @google_data_session.logged_in?
  end
    
  protected
  
  def authorize
    @google_data_session = GoogleDataSession.new(session, create_session_url)     
    logger.debug("session : #{session.inspect}")
  end

  
  def load_spec specification_id
    # File Share Spec: 0AU5sFuLRpCpBZGZra2pqY2pfNDIxZjdkZnM5ZGo -- unauthorised
    if Rails.env =~ /development|test|cucumber/ and specification_id == 'test' 
      show_test
      return
    end
    
    if !logged_in?
      logger.debug "Redirecting to google"
      session[:path_after_login] = request.request_uri
      redirect_to :root
    else
      try_loading_spec(specification_id)
    end
    
  end
  
  def try_loading_spec(specification_id)
    begin
         @google_doc = GoogleDoc.new(specification_id, @google_data_session)
         if @google_doc.authorized?(session)
           @specification = Specification.new(:google_doc => @google_doc)
         else
           flash[:error] = "You don't have permission to view that document! Contact the owner to have it shared with you. (#{@google_doc.id})"            
           redirect_to :root
           return false 
         end
      #rescue Exception=>e
       #  render :text=> e.to_s
     end
  end
  
  def show_test
    require 'mocha'
    GoogleDoc.mock_test!
    GoogleDoc.any_instance.stubs('authorized?').returns(true)
    try_loading_spec('installer_cached')
  end
  def find_use_case use_case_id
    @specification.use_cases.find{|use_case| use_case.to_param == use_case_id}
  end
  
  def load use_case_id, journey_id, use_case_step_id = nil
    @use_case = find_use_case(params[use_case_id])
    if @use_case.nil?
      flash[:error] = "Could not find use case '#{params[use_case_id]}'"
      redirect_to @specification and return
    end
    @journey = @use_case.find_journey_by_id(params[journey_id]) || @use_case.journeys.first
    @use_case_step = @use_case.find_step(params[use_case_step_id]) || @journey.first_step
    @journey.current_step = @use_case_step
    @use_case_map = UseCaseMap::Map.new(:use_case=>@use_case, :journey=>@journey, :specification=>@specification)
  end

  
end
