class GoogleDataSession

  attr_reader :token, :secure, :email, :domain, :client, :create_session_url

  DOC_LIST_URL = "https://docs.google.com/feeds/documents/private/full"
  
  def initialize(session, create_session_url)
    @token                = session[:token]
    @create_session_url   = create_session_url
    @email                = session[:email]

  end
  
  def client
    return @client unless @client.nil?
    @client = GData::Client::DocList.new
    @client.authsub_token = @token
    @client
  end
  
  def scope(domain)
      [
        "https://docs.google.com/feeds/",
        "https://docs.google.com/#{domain}feeds/"
      #,"http://www.google.com/m8/feeds/contacts/default/full"
      ].uniq
  end

  def get_remote_docs
    remote_docs_xml = client.get(DOC_LIST_URL).to_xml
    #render :text=>remote_docs_xml
    remote_docs_xml.elements.to_a('entry').map{|xml| RemoteDoc.new(xml)}.select{|doc| doc.type=='document'}
  end

  def get_document(path)
    get("https://docs.google.com/#{google_account_domain}feeds/#{path}")
  end

  def get(url)
    begin
      client.get(url)
    rescue Exception=>e
      raise "Could not load document #{url} \n(#{e})"
    end
  end
  def authenticate!(token_from_google)
    @client = GData::Client::DocList.new
    @client.authsub_token = token_from_google
    raise "No token found in #{session.inspect} (#{token_from_google})" if token_from_google.blank?
    @token = @client.auth_handler.upgrade()
    @client.authsub_token = @token if @token
    raise "Token was not upgrated! #{@token}" unless @token
    @email = Hpricot(@client.get(DOC_LIST_URL).to_xml.to_s).at('email').to_plain_text
  end
  
  def google_account_domain
    ( email =~ /.+@canonical.com$/ ) ? "a/canonical.com/" : ""
  end
  
  def logged_in?
    !@token.nil?
  end
  
  
  def authsub_url(domain)
    next_url = create_session_url
    secure = APP_CONFIG['secure_gdata']  # set secure = true for signed AuthSub requests
    sess = true
    GData::Auth::AuthSub.get_url(next_url, scope("a/#{domain}/").join(" "), secure, sess, domain)
  end

end