require 'hpricot'
class GoogleDoc
  attr_accessor :html, :txt, :use_cases, :title, :features, :introduction
  ERROR = "Error"
  def initialize(google_doc_id, google_data_session)
    @google_data_session = google_data_session
    @use_cases = []
    @features = []
    unless google_doc_id.nil?
      @google_doc_id = google_doc_id
      refresh! if first_load_necessary?
      @html = File.read( file_name_for :html )
      #@txt  = File.read( file_name_for :txt )
      hpricot = Hpricot clean_html
      @title = GoogleDoc.title hpricot
      parse_introduction! hpricot
      parse_use_cases! hpricot
      parse_steps! hpricot, self
      @raw = hpricot
    end
  end
  def inspect
    "<GoogleDoc id=#{@google_doc_id} use_cases=#{@use_cases.length}>"
  end
  def self.title hpricot
    raw_title = (hpricot.at('#pp2s font') || hpricot.search('h1')[1] || hpricot.search('h1')[0]) 
    raw_title.nil? ? "Title not found" : raw_title.to_plain_text 
  end
  
  def parse_introduction! hpricot
    @introduction = ""
    has_matched_contents_table = false
    toc = hpricot.at("*/div[@class='writely-toc']")

    elem = toc
    until elem.nil? or (elem = elem.next).to_s.downcase =~ /\d\.[^i]+interaction architecture/ do
      @introduction += elem.to_s unless (!elem.nil? and elem.to_plain_text =~ /use-case-mapper\.canonical\.com|(^View.+(with|using))/)
    end
    unless elem.nil?
      @introduction += "<div id='debugging_disclosure'><a href='#debugging_disclosure' title='Click here if the document is missing some content. This will help you repair the Google Doc, which terminates introduction content at the html element containing the text \"Interaction architecture\". Try putting a closing #{elem.name} tag before this heading.'><img src='/images/debug.png'/></a></div>"
      @introduction += "<div class='notice' id='debugging'><p><b>Terminating element:</b> Try placing a closing &lt;/#{elem.name}&gt; tag before the heading \"Interaction architecture\" if any content is not visible here.</p>#{elem}</div>"
    else
      @introduction += "<div class='error'>Didn't find 'Interaction architecture' section - too much introduction may be visible.</div>"
    end
    
  end
  
  def parse_use_cases! hpricot

    for elem in hpricot/'*'

      if (elem.name == 'h3' and !elem.bogusetag? )
        @use_cases.pop unless @use_cases.last and @use_cases.last.confirmed?
        
        use_case = UseCase.new(:specification=>self)
        use_case.href = (elem.at('a') || {} )[:name]#.at('name')
        use_case.title = elem.inner_text.strip
        use_case.title = elem.next.inner_text.strip if use_case.title.blank?
        use_case.title = "#{ERROR} in #{elem.name} #{elem.inspect}" if use_case.title.blank?
        @use_cases << use_case
      end

      if @use_cases.last and elem.name=="b" and elem.to_plain_text.downcase =~ /primary scenario/
        @use_cases.last.debug = "Confirmed by #{elem.to_plain_text}"
        @use_cases.last.confirm!
      end

     end
    @use_cases.compact!
  end

  def parse_steps! hpricot,doc
#    lines = hpricot.search("*").map(&:to_plain_text).to_s.to_a.map(&:strip)
    #raise "how many hpricot lines? #{lines.length} #{lines[0..100].inspect}"
    lines = doc.hpricot_txt.to_a

    @use_cases.each{ |use_case| 
      use_case.find_line_index( lines )
    } 
    @use_cases.each_with_index do |use_case, index|
      next_use_case = use_cases[index+1]
      use_case.add_steps lines.slice(use_case.line_index..( (next_use_case.nil?? lines.length : next_use_case.line_index ) -1 ))

      #use_case.all_steps.each do |step|
      #  elements = hpricot.search("*").select{ |e| e.inner_text == step.last_line  }
      #  step.image = elements.inspect
      #end

    end


  end
  
  def first_load_necessary?
    return true unless File.exists?(file_name_for 'html')
    return false
  end
  
  def hpricot_txt
    @hpricot_txt ||= create_hpricot_text
  end
  def clean_html
    @html.gsub(/<hr class=pb[^>]*>/,'').gsub(/<hr class=\"pb[^>]*>/,'').gsub('<br>','').gsub("&nbsp;"," ")
  end
  def create_hpricot_text
    body = Hpricot(clean_html).search("body")
    #body.search(".writely-comment").remove
    body.inner_text.map(&:strip).map(&:nil_if_blank).compact.join("\n")
  end
  
  def read format
    return html if format == 'html'
    return txt if format == 'txt'
  end
  
  def refresh!
    @html = save_doc_from_server :html
    #@txt = save_doc_from_server :txt
    #File.open(file_name_for(:txt),'w') do |file|
    #  file << get_hpricot
    #end
    
  end
  def get_hpricot
    Hpricot( read('html')).search("*").flatten.map(&:to_plain_text).map(&:strip)
  end
  
  
  def save_doc_from_server format
    #https://docs.google.com/a/canonical.com/feeds/download/documents/Export?docID=0AblM3dK0felBZGZjMnNmY2tfMTk1aGY2M2RmZGo&amp;exportFormat=html
    response = @google_data_session.get_document("download/documents/Export?docID=#{@google_doc_id}&exportFormat=#{format}")
    raise "failed to download file from google docs #{response.inspect}" if response.status_code != 200# and response.status_code != 302
    File.open(file_name_for(format),'w') do |file|
      file << response.body.gsub(/img src=([^\.]+)([^\s]+) /,'img src="https://docs.google.com:443/File?id=\1" ' )
    end
    response.body
  end
  
  def to_s
    response.body
  end
  def original
    GoogleDoc.original(@google_data_session.google_account_domain,@google_doc_id)
  end
  def self.original(domain,id)
    "https://docs.google.com/#{domain}Doc?docid=#{id}"
  end
  def to_txt
    hpricot_txt
  end
  def to_source_html
    read('html')
  end
  
  
  def id
    @google_doc_id
  end

  def to_param
    @google_doc_id
  end
  
  def private?
    false
  end
  
  def authorized?(session)
    session[:authorized_docs] ||= []
    return true if session[:authorized_docs].include?(@google_doc_id)
    begin
       # THIS WOULD BE CALLED EVERY CLICK IF NOT CACHED IN session[:authorized_docs] ! 
       save_doc_from_server 'html'
       session[:authorized_docs] += [@google_doc_id] 
       true
    #rescue
    #    false
    end
  end
  
  def use_cases_for_release(release)
    use_cases.select{ |use_case| use_case.target_release == release.name }
  end
  def use_cases_by_required_feature(required_feature)
    use_cases.select{ |use_case| use_case.required_features.include?(required_feature.name) }
  end
  
  
  def self.mock_test!
    GoogleDoc.any_instance.stubs('first_load_necessary?').returns(false)
    GoogleDoc.any_instance.stubs('cache_path').returns("test/fixtures")
  end
  
  private
  def file_name_for format
    "#{cache_path}/#{@google_doc_id}.#{format}"
  end
  def cache_path
    "tmp/cache"
  end
  
  
end
