class Step
  attr_reader :number, :parent, :children,:small_number
  attr_reader :use_case
  attr_accessor :title, :track,:images, :kind, :videos, :reconnection, :image_titles, :export_images
  
  def initialize(options)
    @children = []
    @use_case = options[:use_case]
    @number= options[:number]
    @parent = options[:parent]
    @title = options[:title].to_s
    @small_number = options[:small_number] || 0
    @kind = options[:kind]
    @in_journey = options[:in_journey]
    @images = []
    @image_titles = []
  end
  def <=>(step)
    if step.kind == :primary and step.number.to_i == number.to_i and self.is_in_variation?
      -1
    elsif self.kind == :primary and step.number.to_i == number.to_i and step.is_in_variation?
      1
    else
      abs_number.to_i<=>step.abs_number.to_i
    end
  end
  def is_in_variation?
    [:reconnect, :variation_child, :junction, :termination].include?(kind)
  end
  #{ |a,b| a.number.to_i == b.number.to_i ? b.abs_number <=> a.abs_number : a.abs_number <=> b.abs_number  }

  def to_param
    number.to_s.gsub(".","_")
  end

  def title_without_number
    title.sub(number.to_s, '').gsub(/^\.\s*/,'')
  end
  def image(index)
    return @images[index][:image] if @images[index]
    ""
  end
  def has_image?
    !image(0).blank?
  end
  def short_description
    "#{number} : #{kind}"
  end
  
  def has_multiple_images?
    !@image_titles.empty?
  end
  
  def add_detail text

    found_image, src = *text.match(/^img:'(.+)'/)
    found_video, video = *text.match(/^video:(.+)/)
    found_multiple_images, image_title = *text.match(/^imgs:(.+)/)
    
    if bug.nil?
      found_bug, @bug_number = *text.match(/\[.*\#(\d+)\]$/) # must be at end of the line
    end
    
    text.gsub!( found_bug.to_s, "" ) if found_bug
    
    marked_done = text =~ /\[done\]$/
    @marked_done = true if marked_done
    
    @images << {:image=>src, :video=>video} if found_image or found_video
    
    @image_titles << image_title if found_multiple_images

    text = "" if text.downcase.strip =~ /^wireframe$|^design$|^implementation$/
    text = text.gsub("[done]","") if @marked_done
    unless text.blank? or found_video or found_image or found_multiple_images
      @title +=  text + "<br/>"
    end
    
  end
  def add_child_step text, small_number
    @children << Step.new(:parent=>@parent,:number=>"#{@number}.#{small_number}",:use_case=>@use_case,:small_number=>small_number, :kind=>:variation_child )
    @children.last.add_detail "#{text}"
    @children.last
  end
  def primary?
    @parent.nil?
  end
  
  def unique_id
    "step_#{@use_case.number}.#{@number}"
  end
  
  def track_index
    has_letter, letter = *number.to_s.match( /\.(\w+)/)
    has_letter ? "#{letter[0] - 96}" : 0
  end
  
  def abs_number
    number.to_i * 100 + small_number.to_i
  end
  
  def marked_done?
    @marked_done
  end
  
  def bug
    if @bug_number 
      @bug ||= Bug.new(@bug_number)
    else
      nil
    end
  end
  
end
