# It is scary in here.
# HOW IT DOES IT:
# GoogleDoc passes in text lines using add_steps
# map_steps goes through each line switching contexts
# Then the journeys are created: steps first, then connections


class UseCase
  attr_accessor :title, :steps, :connections, :trigger, :preconditions, :target_release, :required_features, :href
  attr_accessor :debug
  attr_reader :line_index, :specification,:raw_title, :journeys, :prototype, :notes, :gesture_type
  
  def initialize(options)
    @specification = options[:specification]
    @steps = [Step.new(:number=>0,:use_case=>self,:title=>'start', :kind=>:start)]
    @connections = []
    @journeys = [Journey.new(:title=>"Primary Scenario")]
  end
  def to_xml
    self.instance_variables \
        .reject{ |variable| [:specification, :journeys,:steps].include? variable  } \
        .map{ |variable| { variable.to_sym => self.instance_variable_get(variable)  } } 
      
  end
  def to_param
    "#{number.gsub('.','_')}"
  end

  def find_journey_by_id id
    journeys.find{ |journey| journey.to_param == id } 
  end
  
  def find_line_index lines
    begin 
      matching_line = lines.reverse.find_index{|line| line.match(Regexp.escape(title))}
      @line_index = lines.length - matching_line
      #@title += " (#{@line_index})"
    rescue
      #raise "No line matching '#{title}' in #{lines.length} lines of \n#{lines}"
      matching_line = lines.length - 1
    end
    if @title =~ /^Error/
      @title = "Failed to parse use case title: #{title}"
      @line_index = lines.length - 1
    elsif @line_index.nil?
      @title = "Error parsing use case - no line matching '#{title}'"
      @line_index = lines.length - 1
    end
    @line_index
  end
  
  def add_steps lines
    begin
      lines = lines.map(&:strip)
      @trigger = table_value(lines, /trigger/)
      @preconditions = table_value(lines, /preconditions/)
      @target_release = table_value(lines, /^targeted to land by/) || table_value(lines,/^iteration/) || "No release target"
      @required_features = (table_value(lines, /^required features/)||"").split(",").map(&:strip)#.map{ |f| RequiredFeature.new(:name=>name) }
      
      @steps[0].title = @trigger
      @prototype = table_value(lines, /prototype/)
      @notes = table_value(lines, /notes/)
      @gesture_type = table_value(lines, /gesture type/)
      
      primary_scenario_index = lines.find_index{ |line| line.downcase =~ /primary scenario/ }
      raise "Couldn't match in #{lines}" if primary_scenario_index.nil?
      map_steps lines[(primary_scenario_index+1)..10000]
      
      
      raise "no steps! #{lines}" if steps.length == 1
      
    rescue Exception=>e
      #raise e
      steps << Step.new(:number=>0,:use_case=>self, :kind=>:error)
      steps.last.add_detail "PARSE_ERROR! #{e.to_s}"
    end
    
    create_journeys!
    
    #steps.each{ |step| raise "no sub-steps in #{step.title}!" if step.children.empty? unless step.number == 0 }
  end
  def table_value(lines,search)
    index = lines.find_index{ |line| line.downcase =~ search} 
    index.nil?? nil : lines[index + 1].to_s.strip
  end
  def map_steps lines
    context = :primary
    lines.each do |line|
      case context
        when :primary then context = parse_primary line
        when :variation then context = parse_variation line
        when :variation_step then context = parse_variation_step line
        when :variation_child_step then context = parse_variation_child_step line
        when :terminated then context = :terminated
      end
    end
    raise "never found 'USE CASE ENDS' in #{lines}" if context == :primary
  end
  
  def parse_primary line
    next_index = steps.length
    if (line.match("^#{next_index}")) # 1.   Ubuntu tries to ...
      steps << Step.new(:number=>next_index, :use_case=>self, :kind=>:primary, :in_journey=>true)
      @connections << Connection.new(:from=>steps[-2],:to=>steps[-1], :kind=>:primary, :selected=>true)
    end
    steps.last.add_detail line unless line.empty? or steps.last.nil?
    (line == "USE CASE ENDS") ? :variation : :primary # this is a CONTEXT return
  end

  def parse_variation line
    variation, parent, letter = *line.match( /^(\d+)\.(\w)[\s ]+Variation/) # "3.a   Variation - No wireless ..."
    if !variation.nil?  
      steps << Step.new(:number=>"#{parent}.#{letter}",:parent=>parent, :use_case=>self, :kind=>:junction)
      steps.last.add_detail line
      @journeys << Journey.new(:junction=>steps.last)
      @connections << Connection.new(:from=>steps[steps.last.parent.to_i - 1], :to=>steps.last, :kind=>:junction)
      return :variation_step
    end
    return :variation
    
  end
 
  def parse_variation_step line
    variation_step, number, text = *line.match(/^\.(\d+)(.*)/) # lines starting with ".1 Ubuntu displays a ..."
    termination = line.match(/USE CASE ENDS/)
    if !variation_step.nil?
      steps.last.add_child_step(text, number)
      @connections << Connection.new( :from=>steps.last.children[-2]||steps.last, 
                                      :to=>steps.last.children.last, 
                                      :kind=> :variation  )
      return termination ? :variation_step : :variation_child_step
    end
    
    return parse_variation line
    
  end
  
  def parse_variation_child_step line
    #debugger
    step_child_match = line.match(/\w+/)
    return_to, step_number = *line.match(/RETURN TO (\d+)/)
    termination = line.match(/USE CASE ENDS/)
    variation_step, number, text = *line.match(/^\.(\d+)(.*)/) # lines starting with ".1 Ubuntu displays a ..."
    
    if termination
      steps.last.children.last.kind = :termination
      return :variation_step
    end

    if step_child_match and return_to.nil? and variation_step.nil?
      steps.last.children.last.add_detail(line)
      return :variation_child_step
    end
    
    if return_to
      #steps.last.children.last.add_detail(line)
      return_to_step = steps.find{ |step| step.number.to_i == step_number.to_i and step.primary? }
      if return_to_step.nil?
        steps.last.children.last.add_detail "<br/><em>ERROR: Return-to step (#{step_number}) not found. Returning to branching step instead.</em>"
        return_to_step = steps.last
      end
      reconnection =  Connection.new(:from=>steps.last.children.last, 
                                     :to=>return_to_step,
                                     :kind=>:reconnect)
      @connections << reconnection
      unless termination
        steps.last.children.last.kind = :reconnect
        steps.last.children.last.reconnection = reconnection
      end
      
      #steps.last.children.last.add_detail("Connects #{@connections.last.from.number} to #{@connections.last.to.number}")
      return :variation_step
    end
 
    
    return parse_variation_step line
    #if result
    #  return result
    #else
    #  steps.last.children.last.add_detail line
    #end
    
  end
  
 
  def create_journeys!
    @journeys.first.steps = steps.select{ |step| step.kind==:primary or step.kind ==:start }
    @journeys[1..-1].each do |journey|
      journey.steps = steps_including journey.junction
    end
    @journeys.each{|journey| journey.connections = get_active_connections(journey)}
  end
 
  def has_siblings_for? use_case_step
    return steps.find{ |step| step.parent.to_i == use_case_step.number.to_i}
  end

  def siblings_for use_case_step
    return [use_case_step] + steps.find_all{ |step| step.parent.to_i == use_case_step.number.to_i }
  end

  def confirm!
    @confirmed = true
  end
  def confirmed?
    @confirmed
  end
  
  def title_without_number
    title.gsub(number,"").strip
  end
  
  def number
    match, number = *title.match(/(\d+\.\d+(\.[A-Z])?)/)
    number || "0"
  end
  
  def all_steps
    @all_steps ||= (steps + steps.map{|step| step.children}.flatten).sort_by(&:abs_number)
  end
  def find_step(number)
    #all_steps.find{ |step| step.to_param == number }
    number = number.to_s.gsub("_", ".")
    result = all_steps.find{ |step| step.number.to_s == number.to_s  }
    if result and result.kind == :junction
      result.children.first
    else
      result
    end
  end
  def groups
    @groups ||= steps.map{ |step| 
      siblings = steps.find_all{ |s| s.number.to_i == step.number.to_i}
        {
          :step=>step,
          :number=>step.number, 
          :siblings=>siblings,
          :max_children=> siblings.max{ |a,b| a.children.length <=> b.children.length }.children.length
         } if step.primary?
    }.compact
  end
  
  def next
    @next ||= specification.use_cases[specification.use_cases.find_index{ |use_case| use_case==self } + 1]
  end
  def previous
    @previous ||= specification.use_cases[specification.use_cases.find_index{ |use_case| use_case==self } - 1]
  end
   
  def steps_including journey_step
    terminated = false
    reconnection = nil
    result = []
    #puts "******* JOURNEY INCLUDING #{journey_step.short_description} *********"
    all_steps.sort.each{ |step|
      
      in_same_variation = (step.track_index == journey_step.track_index) and (!step.number.blank?)
      is_a_variation = step.number.to_i == journey_step.number.to_i
      
      
      
      result << if step.primary? and terminated
        
        ignore step, :because => "After termination step (#{terminated.short_description})", :in=>journey_step
        
      elsif step.primary? and reconnection and step.number.to_i < reconnection.to.number.to_i 
        #puts "  reconnection.to = #{reconnection.to.short_description}  > #{step.short_description} ? "
        ignore step, :because => "Between branched primary step and return-to step", :in=>journey_step
        
      elsif step.primary? and !is_a_variation
        
        append step, :because => "step.primary? and !is_a_variation", :in=>journey_step
        
      elsif is_a_variation and in_same_variation
      
        append step, :because => "is_a_variation and in_same_variation" , :in=>journey_step

      elsif !step.primary?

        ignore step, :because => "Non-primary step", :in=>journey_step

      else
        
        append step, :because => "Not filtered by any other condition", :in=>journey_step
        
      end
      terminated = step if !terminated and step.kind == :termination and result.include?(step)
      if reconnection.nil? and in_same_variation and step.kind == :reconnect and result.include?(step)
        #puts "Found a reconnection step (#{step.short_description}) [#{step.track_index} , #{journey_step.track_index}]"
        reconnection = step.reconnection
      end
    }
    result.flatten.uniq.compact.sort # SORTING TWICE..... OPTIMISATION POSSIBLE?
  end
  
  DEBUGGING  = false
  
  def ignore step, details
    DEBUGGING ? (step.title += "<br/>Journey: #{details[:in].number}<br/><em>[IGNORED: #{details[:because]}]</em>" ; step) : nil
  end
  
  def append step, details
    DEBUGGING ? (step.title += "<br/>Journey: #{details[:in].number}<br/><em>[INCLUDED: #{details[:because]}]</em>"; step) : step
  end
  
  def get_active_connections journey
    result = []
    
    @connections.each do |connection|
      case connection.kind
      when :primary
        is_skipped = !(journey.steps.include?(connection.to) and journey.steps.include?(connection.from) )
        result << connection unless is_skipped
      when :reconnect
        result << connection if journey.steps.include?(connection.from)
      else
        result << connection if (journey.steps.find{|step| step == connection.to} != nil)
      end
    end
    result = result.group_by(&:from).map{ |from,c| c.last }
    
    return result
  end
  
  def primary_scenario
    steps.map{ |step| step if step.primary? }.compact
  end
  
end
