module UseCaseMap
  class Map
    SPACING = 22.0
    attr_reader :proxy, :points, :lines, :total_height, :total_width
    def initialize(options)
      @specification = options[:specification]
      @use_case = options[:use_case]
      @journey = options[:journey]
      @horizontal = options[:horizontal]
      @proxy = create_proxy options[:use_case]
    end
    def create_proxy use_case
      group_index = 0 
      points = use_case.groups[1..-1].map{ |group| 
        #group_index += 1 if group[:siblings].length > 1 
        result = (group[:siblings] + group[:siblings].map(&:children).flatten).map{ |sibling|
          Point.new({
            :sibling=>sibling,
            :spacing=>spacing,
            :group_index=> group_index + ( (sibling.track_index.to_i == 0 and  group[:siblings].length > 1) ? 1 : 0),
            :title=>sibling.kind==:junction ? sibling.number: sibling.number.to_s.split(".").last,#title,
            :href=> @journey.nil? ? "" : [
                              @specification, 
                              @use_case, 
                              @journey.steps.include?(sibling) ? @journey : @use_case.journeys.find{ |journey| journey.steps.include?(sibling) }, 
                              sibling
                              ].map{ |o| "/#{o.class.to_s.tableize}/#{o.to_param}" }.join(""),
            :children=>sibling.children.length,
            :image=> sibling.images.first.nil? ? "" : sibling.images.first[:image],
            :unique_id=>sibling.unique_id,
            :kind=>sibling.kind,
            :selected=> @journey.nil? ? false : @journey.current_step == sibling,
            :in_journey=> @journey.nil? ? true : @journey.steps.include?(sibling),
            :bug=>sibling.bug,
            :marked_done=>sibling.marked_done?,
            :horizontal=>@horizontal
          })
        }
        group_index += group[:max_children] + 1
        result
      }.flatten
  
      @total_height = points.max{ |a,b|a.y<=>b.y}.y + spacing
      @total_width  = points.max{ |a,b|a.x<=>b.x}.x + spacing
  
      @lines = use_case.connections.map{|connection|
        # find the points
        {
          :from=> points.find{|step| step.unique_id == connection.from_unique_id},
          :to=>   points.find{|step| step.unique_id == connection.to_unique_id},
          :kind=> connection.kind,
          :selected=> @journey.nil? ? true : @journey.connections.include?(connection)
        }
        # plot the points
      }.select{ |line| line unless line[:from].nil? or line[:to].nil?  }.map{|connection|
        {
            :from=>{ :x=>connection[:from].x, :y=>connection[:from].y },
            :to=> { :x=>connection[:to].x, :y=>connection[:to].y },
            :kind=>connection[:kind],
            :selected=>connection[:selected]
        }
      }
    
      @points = points.flatten
    
    end
    def lines_json
      @lines.map(&:to_inline_json).join(',')
    end
    def points_json
      @points.map(&:to_inline_json).join(",")
    end
    def spacing
      @spacing || SPACING
    end
    def rows
      @points.group_by{ |item| item[:y]  }
    end
    def canvas_id
      "map_canvas_#{@use_case.to_param}"
    end
  
  end
  class Point
    def properties
      [:x,:y,:title,:href,:children,:image,:unique_id,:kind,:selected,:in_journey,:bug,:marked_done]
    end
    def initialize(options)
      @options = options
      @json = Hash[properties.map{ |p| [p,self.send(p)] }].to_inline_json
    end
    def method_missing name
      @options[name]
    end
    def x
      horizontal ? step : track
    end
    def y
      horizontal ? track : step
    end
    def track
      spacing + sibling.track_index.to_i * (spacing + 6)
    end
    def step
      spacing + group_index * spacing + @options[:sibling].small_number.to_i * @options[:spacing]
    end
    def to_inline_json
      @json
    end
  end
  class Line

  end
end
