var COLOURS = {	primary:   "#93B8DA", // light blue
				junction:  "#0000FF", // blue
				reconnect: "#FF0000", // red
				in_journey:"#796646", // purple 
				selected_line:"#796646", // purple
				deselected_line:'#D3CAAA', // light
				selected_point:'#f15a29', // orange
				deselected_point:'#D3CAAA', // light
				marked_done: "#538f53", // green
				bug: "#e12626" // red
			    };

function getColoursAsListItems(){
	result = "";
	for (i in COLOURS) {
		result += "<li style='color:"+COLOURS[i]+";'>" + i + "</li>";
	};
	return result;
}
var MapDrawer = function(options){
	
	var mouseInfo = {x:0,y:0};
	var POINT_SIZE = 8;
	var LINE_WEIGHT = 3;
	
	var canvas_id;
	var canvas;
    var ctx;

	var lines;
    var points;
	
	var point_under_mouse;
	var clicked_point;
	
	function init(){
	    lines = options.lines;
	    points = options.points;
	    spacing = options.spacing;
	    canvas_id = options.canvas_id;
	    canvas = document.getElementById(canvas_id);
	    ctx = canvas.getContext("2d");
		draw();
		$(canvas).mousemove(onCanvasMouseMove).click(onCanvasClick);
	}
	function draw(){
		ctx.clearRect(0,0,canvas.width,canvas.height); 
		drawLines(lines, 'deselected_line');
		drawPoints({state:'inactive'} );		
		drawLines(getSelectedLines(),'selected_line')
        drawPoints({state:'active'})
	}
	function getSelectedLines(){
		var result  = [];
		for (var i=0; i < lines.length; i++) {
			var line = lines[i];
			if(line.selected) result.push(line);
		};
		//alert("lines:" + lines.length + ", selected lines:" + result.length)
		return result;
	}
	function rect(rect){
	    ctx.beginPath();
	    ctx.rect(rect.x,rect.y,rect.width,rect.height);
	    ctx.fill();
	}
	function drawConnector(lines){
	    for (var i=0; i < lines.length; i++) {
	       var line = lines[i];
	       rect(line);
	    }
	}
	function drawLines(lines, colour){
	    ctx.lineWidth = LINE_WEIGHT;
        ctx.strokeStyle = getColour(colour);
        
        for(var i = 0; i < lines.length; i++){
            var line = lines[i];
    		//var split = {x:line.to.x - line.from.x, 
    		//                 y:line.to.y - line.from.y - spacing};
            //// if(line.kind == 'reconnect') split.y += spacing;
            //var vertical = {x: line.from.x - LINE_WEIGHT / 2, 
    		//                y: line.from.y, 
    		//                width: LINE_WEIGHT, 
    		//                height: split.y};
    		//var horizontal = {x:line.from.x - LINE_WEIGHT / 2, 
            //        		  y:line.from.y + split.y, 
            //        		  width:split.x, 
            //        		  height:LINE_WEIGHT};
            //var group_connector = {x:line.from.x + split.x - LINE_WEIGHT / 2, 
            //          		       y:line.from.y + split.y, 
            //          		       width:LINE_WEIGHT, 
            //          		       height:spacing};
            //ctx.fillStyle = getColour(line.kind);//colour) ;//getColour(line.kind);
            //ctx.lineWeight(LINE_WEIGHT);
            // drawConnector( line.kind == 'reconnect' ?
            //                  [horizontal,vertical] :
            //                  [vertical,horizontal,group_connector]);
            ctx.beginPath();
            //ctx.strokeStyle = getColour(line.kind);
            ctx.moveTo(line.from.x,line.from.y);
            ctx.lineTo(line.to.x,line.to.y);
            ctx.stroke();
	    }
		
	}
	function getColour(kind){
		return COLOURS[kind] || "#cccccc";
	}
	function drawPoints(options){
	    var state = options['state'];
		for (var i=0; i < points.length; i++) {
			var point = points[i];
            if((state == 'active' && point.in_journey) || (state == 'inactive' && !point.in_journey) ){
    			ctx.beginPath();
    			ctx.fillStyle = getColour(point == point_under_mouse ? "#D0D38E" : "deselected_point");
    			if (point.in_journey == true) ctx.fillStyle = getColour("in_journey");
    			if (point.marked_done) ctx.fillStyle = getColour('marked_done');
    			if (point.bug) ctx.fillStyle = getColour('bug');
    			if (point.selected) ctx.fillStyle = getColour('selected_point');
			
                // ctx.fillStyle = getColour(point.kind);
			
    			drawPoint(point)
                ctx.fill();

    			ctx.beginPath();
    			ctx.textAlign = "center"
    			ctx.fillStyle = "#FFFFFF";
    			ctx.fillText(point.title, point.x, point.y + 3);
    			ctx.fill();
			}
		};
	}


	function drawPoint(point){
		if(point.kind == 'junction'){ // VARIATION
			ctx.rect(point.x-14, point.y-6, 26,12)
			return;
		}
		if(point.image){ // IMAGE 
			ctx.rect(point.x - 8, point.y - 6, 16, 12); return;
		}else{	// CIRCLE
			ctx.arc(point.x,point.y,8,0, Math.PI*2,false );
		}

	}
	function onCanvasClick(event){
		var p = getPointUnderMouse(event,this) ;
		if(!p) return;
		clicked_point = p;
		draw();
		
		//debugger
		$("#journey_spinner").show().css({left:clicked_point.x + this.offsetLeft - 5,top:clicked_point.y + this.offsetTop- 6});
		$("#tooltip").fadeOut();
		//alert("clicked_point:" + clicked_point.href);
	    
		replaceJourneyAndWireframe(clicked_point.href);

	}
	
	function onCanvasMouseMove(event){
		point_under_mouse = getPointUnderMouse(event,this);

		$(this).css('cursor', point_under_mouse ? 'pointer' : 'default');
		if(event.x == event.pageX && mouseInfo && mouseInfo.y == event.pageY){
				mouseInfo.clicked = point_under_mouse;
				return;
		}else{
			if(clicked_point  && point_under_mouse){
				//debugger
				if (clicked_point.unique_id == point_under_mouse.unique_id) return;
			} 
			mouseInfo = {x: event.pageX, y: event.pageY };
			clicked_point = null;
		}
		
		draw();
		// var tooltip = $("#tooltip");
		// if(point_under_mouse){
		// 	tooltip.show().html(point_under_mouse.title ).css({left:(event.clientX + 20) + 'px',top:(event.pageY - 20) + 'px'})
		// }else{
		// 	tooltip.hide();
		// }
	}
	
	function getPointUnderMouse(event, receiver){
		var x = event.pageX - 270; // HACK BASED ON COLUMN WIDTH IN APPLICATION.CSS
		var y = event.pageY - receiver.offsetTop;

		var point_click_size = POINT_SIZE;
		for (var i=0; i < points.length; i++) {
			var point = points[i];
			if(	x >= point.x - point_click_size && 
					x <= point.x + point_click_size && 
					y >= point.y - point_click_size && 
					y <= point.y + point_click_size	&& 
					point.href != ""){
						
			return point;
		 }
		};
		return null;
	}
	
	this.update = function(new_lines,new_points){
		lines = new_lines;
		points = new_points;
		draw();
	}
	
	init();
}
